@@ -95,11 +95,11 @@
 RecordBatchReader <- R6Class("RecordBatchReader", inherit = ArrowObject,
   public = list(
     read_next_batch = function() {
-      shared_ptr(RecordBatch, RecordBatchReader__ReadNext(self))
+      RecordBatchReader__ReadNext(self)
     }
   ),
   active = list(
-    schema = function() shared_ptr(Schema, RecordBatchReader__schema(self))
+    schema = function() RecordBatchReader__schema(self)
   )
 )
 
@@ -109,8 +109,8 @@ RecordBatchReader <- R6Class("RecordBatchReader", inherit = ArrowObject,
 #' @export
 RecordBatchStreamReader <- R6Class("RecordBatchStreamReader", inherit = RecordBatchReader,
   public = list(
-    batches = function() map(ipc___RecordBatchStreamReader__batches(self), shared_ptr, class = RecordBatch),
-    read_table = function() shared_ptr(Table, Table__from_RecordBatchStreamReader(self))
+    batches = function() ipc___RecordBatchStreamReader__batches(self),
+    read_table = function() Table__from_RecordBatchReader(self)
   )
 )
 RecordBatchStreamReader$create <- function(stream) {
@@ -120,7 +120,7 @@ RecordBatchStreamReader$create <- function(stream) {
     stream <- BufferReader$create(stream)
   }
   assert_is(stream, "InputStream")
-  shared_ptr(RecordBatchStreamReader, ipc___RecordBatchStreamReader__Open(stream))
+  ipc___RecordBatchStreamReader__Open(stream)
 }
 
 #' @rdname RecordBatchReader
@@ -128,19 +128,20 @@ RecordBatchStreamReader$create <- function(stream) {
 #' @format NULL
 #' @export
 RecordBatchFileReader <- R6Class("RecordBatchFileReader", inherit = ArrowObject,
-  # Why doesn't this inherit from RecordBatchReader?
+  # Why doesn't this inherit from RecordBatchReader in C++?
+  # Origin: https://github.com/apache/arrow/pull/679
   public = list(
     get_batch = function(i) {
-      shared_ptr(RecordBatch, ipc___RecordBatchFileReader__ReadRecordBatch(self, i))
+      ipc___RecordBatchFileReader__ReadRecordBatch(self, i)
     },
     batches = function() {
-      map(ipc___RecordBatchFileReader__batches(self), shared_ptr, class = RecordBatch)
+      ipc___RecordBatchFileReader__batches(self)
     },
-    read_table = function() shared_ptr(Table, Table__from_RecordBatchFileReader(self))
+    read_table = function() Table__from_RecordBatchFileReader(self)
   ),
   active = list(
     num_record_batches = function() ipc___RecordBatchFileReader__num_record_batches(self),
-    schema = function() shared_ptr(Schema, ipc___RecordBatchFileReader__schema(self))
+    schema = function() ipc___RecordBatchFileReader__schema(self)
   )
 )
 RecordBatchFileReader$create <- function(file) {
@@ -150,5 +151,5 @@ RecordBatchFileReader$create <- function(file) {
     file <- BufferReader$create(file)
   }
   assert_is(file, "InputStream")
-  shared_ptr(RecordBatchFileReader, ipc___RecordBatchFileReader__Open(file))
+  ipc___RecordBatchFileReader__Open(file)
 }
