@@ -23,61 +23,34 @@
 #include <arrow/record_batch.h>
 #include <arrow/table.h>
 
+std::shared_ptr<arrow::compute::CastOptions> make_cast_options(cpp11::list options);
+
 arrow::compute::ExecContext* gc_context() {
   static arrow::compute::ExecContext context(gc_memory_pool());
   return &context;
 }
 
-// [[arrow::export]]
-std::shared_ptr<arrow::compute::CastOptions> compute___CastOptions__initialize(
-    bool allow_int_overflow, bool allow_time_truncate, bool allow_float_truncate) {
-  auto options = std::make_shared<arrow::compute::CastOptions>();
-  options->allow_int_overflow = allow_int_overflow;
-  options->allow_time_truncate = allow_time_truncate;
-  options->allow_float_truncate = allow_float_truncate;
-  return options;
-}
-
-// [[arrow::export]]
-std::shared_ptr<arrow::Array> Array__cast(
-    const std::shared_ptr<arrow::Array>& array,
-    const std::shared_ptr<arrow::DataType>& target_type,
-    const std::shared_ptr<arrow::compute::CastOptions>& options) {
-  return ValueOrStop(arrow::compute::Cast(*array, target_type, *options, gc_context()));
-}
-
-// [[arrow::export]]
-std::shared_ptr<arrow::ChunkedArray> ChunkedArray__cast(
-    const std::shared_ptr<arrow::ChunkedArray>& chunked_array,
-    const std::shared_ptr<arrow::DataType>& target_type,
-    const std::shared_ptr<arrow::compute::CastOptions>& options) {
-  arrow::Datum value(chunked_array);
-  arrow::Datum out =
-      ValueOrStop(arrow::compute::Cast(value, target_type, *options, gc_context()));
-  return out.chunked_array();
-}
-
 // [[arrow::export]]
 std::shared_ptr<arrow::RecordBatch> RecordBatch__cast(
     const std::shared_ptr<arrow::RecordBatch>& batch,
-    const std::shared_ptr<arrow::Schema>& schema,
-    const std::shared_ptr<arrow::compute::CastOptions>& options) {
+    const std::shared_ptr<arrow::Schema>& schema, cpp11::list options) {
+  auto opts = make_cast_options(options);
   auto nc = batch->num_columns();
 
   arrow::ArrayVector columns(nc);
   for (int i = 0; i < nc; i++) {
     columns[i] = ValueOrStop(
-        arrow::compute::Cast(*batch->column(i), schema->field(i)->type(), *options));
+        arrow::compute::Cast(*batch->column(i), schema->field(i)->type(), *opts));
   }
 
   return arrow::RecordBatch::Make(schema, batch->num_rows(), std::move(columns));
 }
 
 // [[arrow::export]]
-std::shared_ptr<arrow::Table> Table__cast(
-    const std::shared_ptr<arrow::Table>& table,
-    const std::shared_ptr<arrow::Schema>& schema,
-    const std::shared_ptr<arrow::compute::CastOptions>& options) {
+std::shared_ptr<arrow::Table> Table__cast(const std::shared_ptr<arrow::Table>& table,
+                                          const std::shared_ptr<arrow::Schema>& schema,
+                                          cpp11::list options) {
+  auto opts = make_cast_options(options);
   auto nc = table->num_columns();
 
   using ColumnVector = std::vector<std::shared_ptr<arrow::ChunkedArray>>;
@@ -85,7 +58,7 @@ std::shared_ptr<arrow::Table> Table__cast(
   for (int i = 0; i < nc; i++) {
     arrow::Datum value(table->column(i));
     arrow::Datum out =
-        ValueOrStop(arrow::compute::Cast(value, schema->field(i)->type(), *options));
+        ValueOrStop(arrow::compute::Cast(value, schema->field(i)->type(), *opts));
     columns[i] = out.chunked_array();
   }
   return arrow::Table::Make(schema, std::move(columns), table->num_rows());
@@ -171,6 +144,33 @@ std::shared_ptr<arrow::compute::FunctionOptions> make_compute_options(
     return out;
   }
 
+  if (func_name == "array_sort_indices") {
+    using Order = arrow::compute::SortOrder;
+    using Options = arrow::compute::ArraySortOptions;
+    // false means descending, true means ascending
+    auto order = cpp11::as_cpp<bool>(options["order"]);
+    auto out =
+        std::make_shared<Options>(Options(order ? Order::Descending : Order::Ascending));
+    return out;
+  }
+
+  if (func_name == "sort_indices") {
+    using Key = arrow::compute::SortKey;
+    using Order = arrow::compute::SortOrder;
+    using Options = arrow::compute::SortOptions;
+    auto names = cpp11::as_cpp<std::vector<std::string>>(options["names"]);
+    // false means descending, true means ascending
+    // cpp11 does not support bool here so use int
+    auto orders = cpp11::as_cpp<std::vector<int>>(options["orders"]);
+    std::vector<Key> keys;
+    for (size_t i = 0; i < names.size(); i++) {
+      keys.push_back(
+          Key(names[i], (orders[i] > 0) ? Order::Descending : Order::Ascending));
+    }
+    auto out = std::make_shared<Options>(Options(keys));
+    return out;
+  }
+
   if (func_name == "min_max") {
     using Options = arrow::compute::MinMaxOptions;
     auto out = std::make_shared<Options>(Options::Defaults());
@@ -179,42 +179,88 @@ std::shared_ptr<arrow::compute::FunctionOptions> make_compute_options(
     return out;
   }
 
+  if (func_name == "quantile") {
+    using Options = arrow::compute::QuantileOptions;
+    auto out = std::make_shared<Options>(Options::Defaults());
+    SEXP q = options["q"];
+    if (!Rf_isNull(q) && TYPEOF(q) == REALSXP) {
+      out->q = cpp11::as_cpp<std::vector<double>>(q);
+    }
+    SEXP interpolation = options["interpolation"];
+    if (!Rf_isNull(interpolation) && TYPEOF(interpolation) == INTSXP &&
+        XLENGTH(interpolation) == 1) {
+      out->interpolation =
+          cpp11::as_cpp<enum arrow::compute::QuantileOptions::Interpolation>(
+              interpolation);
+    }
+    return out;
+  }
+
   if (func_name == "is_in" || func_name == "index_in") {
     using Options = arrow::compute::SetLookupOptions;
     return std::make_shared<Options>(cpp11::as_cpp<arrow::Datum>(options["value_set"]),
                                      cpp11::as_cpp<bool>(options["skip_nulls"]));
   }
 
-  // hacky attempt to pass through to_type and other options
-  if (func_name == "cast") {
-    using Options = arrow::compute::CastOptions;
-    auto out = std::make_shared<Options>(true);
-    SEXP to_type = options["to_type"];
-    if (!Rf_isNull(to_type) && cpp11::as_cpp<std::shared_ptr<arrow::DataType>>(to_type)) {
-      out->to_type = cpp11::as_cpp<std::shared_ptr<arrow::DataType>>(to_type);
+  if (func_name == "dictionary_encode") {
+    using Options = arrow::compute::DictionaryEncodeOptions;
+    auto out = std::make_shared<Options>(Options::Defaults());
+    if (!Rf_isNull(options["null_encoding_behavior"])) {
+      out->null_encoding_behavior = cpp11::as_cpp<
+          enum arrow::compute::DictionaryEncodeOptions::NullEncodingBehavior>(
+          options["null_encoding_behavior"]);
     }
-
-    SEXP allow_float_truncate = options["allow_float_truncate"];
-    if (!Rf_isNull(allow_float_truncate) && cpp11::as_cpp<bool>(allow_float_truncate)) {
-      out->allow_float_truncate = cpp11::as_cpp<bool>(allow_float_truncate);
-    }
-
-    SEXP allow_time_truncate = options["allow_time_truncate"];
-    if (!Rf_isNull(allow_time_truncate) && cpp11::as_cpp<bool>(allow_time_truncate)) {
-      out->allow_time_truncate = cpp11::as_cpp<bool>(allow_time_truncate);
-    }
-
-    SEXP allow_int_overflow = options["allow_int_overflow"];
-    if (!Rf_isNull(allow_int_overflow) && cpp11::as_cpp<bool>(allow_int_overflow)) {
-      out->allow_int_overflow = cpp11::as_cpp<bool>(allow_int_overflow);
-    }
-
     return out;
   }
 
+  if (func_name == "cast") {
+    return make_cast_options(options);
+  }
+
+  if (func_name == "match_substring" || func_name == "match_substring_regex") {
+    using Options = arrow::compute::MatchSubstringOptions;
+    return std::make_shared<Options>(cpp11::as_cpp<std::string>(options["pattern"]));
+  }
+
+  if (func_name == "replace_substring" || func_name == "replace_substring_regex") {
+    using Options = arrow::compute::ReplaceSubstringOptions;
+    int64_t max_replacements = -1;
+    if (!Rf_isNull(options["max_replacements"])) {
+      max_replacements = cpp11::as_cpp<int64_t>(options["max_replacements"]);
+    }
+    return std::make_shared<Options>(cpp11::as_cpp<std::string>(options["pattern"]),
+                                     cpp11::as_cpp<std::string>(options["replacement"]),
+                                     max_replacements);
+  }
+
   return nullptr;
 }
 
+std::shared_ptr<arrow::compute::CastOptions> make_cast_options(cpp11::list options) {
+  using Options = arrow::compute::CastOptions;
+  auto out = std::make_shared<Options>(true);
+  SEXP to_type = options["to_type"];
+  if (!Rf_isNull(to_type) && cpp11::as_cpp<std::shared_ptr<arrow::DataType>>(to_type)) {
+    out->to_type = cpp11::as_cpp<std::shared_ptr<arrow::DataType>>(to_type);
+  }
+
+  SEXP allow_float_truncate = options["allow_float_truncate"];
+  if (!Rf_isNull(allow_float_truncate) && cpp11::as_cpp<bool>(allow_float_truncate)) {
+    out->allow_float_truncate = cpp11::as_cpp<bool>(allow_float_truncate);
+  }
+
+  SEXP allow_time_truncate = options["allow_time_truncate"];
+  if (!Rf_isNull(allow_time_truncate) && cpp11::as_cpp<bool>(allow_time_truncate)) {
+    out->allow_time_truncate = cpp11::as_cpp<bool>(allow_time_truncate);
+  }
+
+  SEXP allow_int_overflow = options["allow_int_overflow"];
+  if (!Rf_isNull(allow_int_overflow) && cpp11::as_cpp<bool>(allow_int_overflow)) {
+    out->allow_int_overflow = cpp11::as_cpp<bool>(allow_int_overflow);
+  }
+  return out;
+}
+
 // [[arrow::export]]
 SEXP compute__CallFunction(std::string func_name, cpp11::list args, cpp11::list options) {
   auto opts = make_compute_options(func_name, options);
@@ -224,4 +270,32 @@ SEXP compute__CallFunction(std::string func_name, cpp11::list args, cpp11::list
   return from_datum(std::move(out));
 }
 
+// [[arrow::export]]
+SEXP compute__GroupBy(cpp11::list arguments, cpp11::list keys, cpp11::list options) {
+  // options is a list of pairs: string function name, list of options
+
+  std::vector<std::shared_ptr<arrow::compute::FunctionOptions>> keep_alives;
+  std::vector<arrow::compute::internal::Aggregate> aggregates;
+
+  for (cpp11::list name_opts : options) {
+    auto name = cpp11::as_cpp<std::string>(name_opts[0]);
+    auto opts = make_compute_options(name, name_opts[1]);
+
+    aggregates.push_back(
+        arrow::compute::internal::Aggregate{std::move(name), opts.get()});
+    keep_alives.push_back(std::move(opts));
+  }
+
+  auto datum_arguments = arrow::r::from_r_list<arrow::Datum>(arguments);
+  auto datum_keys = arrow::r::from_r_list<arrow::Datum>(keys);
+  auto out = ValueOrStop(arrow::compute::internal::GroupBy(datum_arguments, datum_keys,
+                                                           aggregates, gc_context()));
+  return from_datum(std::move(out));
+}
+
+// [[arrow::export]]
+std::vector<std::string> compute__GetFunctionNames() {
+  return arrow::compute::GetFunctionRegistry()->GetFunctionNames();
+}
+
 #endif
